﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using RemoteTestRunner.FormsUI;
using RemoteTestRunner.BusinessLogic;
using RemoteTestRunner.BusinessLogic.Presenters;

namespace FormsUI
{
    public static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        public static void Main()
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            Application.SetCompatibleTextRenderingDefault(false);

            Application.ThreadException += new System.Threading.ThreadExceptionEventHandler(Application_ThreadException);
            AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(CurrentDomain_UnhandledException);

            Runner runnerInstance = new Runner();
            IoC.Container.RegisterInstance<Runner>(runnerInstance);

            IoC.Container.RegisterType<TestRunnerPresenter, TestRunnerPresenter>();
            IoC.Container.RegisterType<RemoteTestRunner.BusinessLogic.Views.ITestRunnerView, Runner>();

            IoC.Container.RegisterType<ApplicationNavigationPresenter, ApplicationNavigationPresenter>();
            IoC.Container.RegisterType<RemoteTestRunner.BusinessLogic.Views.IApplicationNavigationView, Runner>();

            IoC.Container.RegisterType<LongRunningPresenter, LongRunningPresenter>();
            IoC.Container.RegisterType<RemoteTestRunner.BusinessLogic.Views.ILongRunningView, LongRunning>();

            IoC.Container.RegisterType<EditProjectsPresenter, EditProjectsPresenter>();
            IoC.Container.RegisterType<RemoteTestRunner.BusinessLogic.Views.IEditProjectsView, ProjectAdmin>();

            IoC.Container.RegisterType<EditTestsPresenter, EditTestsPresenter>();
            IoC.Container.RegisterType<RemoteTestRunner.BusinessLogic.Views.IEditTestsView, TestAdmin>();

            IoC.Container.RegisterType<OptionsPresenter, OptionsPresenter>();
            IoC.Container.RegisterType<RemoteTestRunner.BusinessLogic.Views.IOptionsView, Options>();

            PresenterBase navigationPresenter = IoC.Container.Resolve<ApplicationNavigationPresenter>();
            navigationPresenter.Start();

            PresenterBase mainPresenter = IoC.Container.Resolve<TestRunnerPresenter>();
            mainPresenter.Start();

            Application.Run();
        }

        private static void CurrentDomain_UnhandledException(object sender, UnhandledExceptionEventArgs e)
        {
            MessageBox.Show("Application exception:\n\n" + e.ExceptionObject.ToString());
        }

        private static void Application_ThreadException(object sender, System.Threading.ThreadExceptionEventArgs e)
        {
            MessageBox.Show("Application exception:\n\n" + e.Exception.ToString());
        }
    }
}
