﻿using System.Windows;
using System.Windows.Controls;

namespace RemoteTestRunner.FormsUI.Wpf
{
    public class TreeListView : TreeView
    {
        protected override DependencyObject
            GetContainerForItemOverride()
        {
            return new TreeListViewItem();
        }

        protected override bool
            IsItemItsOwnContainerOverride(object item)
        {
            return item is TreeListViewItem;
        }
    }

    public class TreeListViewItem : TreeViewItem
    {
        private int _level = -1;

        /// <summary>
        ///   Item's hierarchy in the tree
        /// </summary>
        public int Level
        {
            get
            {
                if (_level == -1)
                {
                    var parent = ItemsControlFromItemContainer(this) as TreeListViewItem;
                    _level = (parent != null) ? parent.Level + 1 : 0;
                }
                return _level;
            }
        }


        protected override DependencyObject
            GetContainerForItemOverride()
        {
            return new TreeListViewItem();
        }

        protected override bool IsItemItsOwnContainerOverride(object item)
        {
            return item is TreeListViewItem;
        }
    }
}